@@ -8,7 +8,7 @@
 
 #include <asm/types.h>		/* For __uXX types */
 
-#define IP_VS_VERSION_CODE	0x010103
+#define IP_VS_VERSION_CODE	0x010104
 #define NVERSION(version)			\
 	(version >> 16) & 0xFF,			\
 	(version >> 8) & 0xFF,			\
@@ -92,6 +92,10 @@
 #define IP_VS_IFNAME_MAXLEN	16
 
 
+/*
+ *	The struct ip_vs_service_user and struct ip_vs_dest_user are
+ *	used to set IPVS rules through setsockopt.
+ */
 struct ip_vs_service_user {
 	/* virtual service addresses */
 	u_int16_t		protocol;
@@ -181,12 +185,14 @@ struct ip_vs_dest_entry {
 	u_int16_t		port;
 	unsigned		conn_flags;	/* connection flags */
 	int			weight;		/* destination weight */
-	u_int32_t		activeconns;	/* active connections */
-	u_int32_t		inactconns;	/* inactive connections */
 
 	u_int32_t		u_threshold;	/* upper threshold */
 	u_int32_t		l_threshold;	/* lower threshold */
 
+	u_int32_t		activeconns;	/* active connections */
+	u_int32_t		inactconns;	/* inactive connections */
+	u_int32_t		persistconns;	/* persistent connections */
+
 	/* statistics */
 	struct ip_vs_stats_user stats;
 };
@@ -370,13 +376,17 @@ enum {
 	IP_VS_TCP_S_LAST
 };
 
-/* UDP State Values */
+/*
+ *	UDP State Values
+ */
 enum {
 	IP_VS_UDP_S_NORMAL,
 	IP_VS_UDP_S_LAST,
 };
 
-/* ICMP State Values */
+/*
+ *	ICMP State Values
+ */
 enum {
 	IP_VS_ICMP_S_NORMAL,
 	IP_VS_ICMP_S_LAST,
@@ -606,6 +616,7 @@ struct ip_vs_dest {
 	/* connection counters and thresholds */
 	atomic_t		activeconns;	/* active connections */
 	atomic_t		inactconns;	/* inactive connections */
+	atomic_t		persistconns;	/* persistent connections */
 	__u32			u_threshold;	/* upper threshold */
 	__u32			l_threshold;	/* lower threshold */
 
