@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.30 2002/11/30 01:50:35 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.31 2003/04/18 09:03:16 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -303,8 +303,8 @@ void ip_vs_conn_fill_cport(struct ip_vs_conn *cp, __u16 cport)
 
 
 /*
- *  Bind a connection entry with the corresponding packet_xmit.
- *  Called by ip_vs_conn_new.
+ *	Bind a connection entry with the corresponding packet_xmit.
+ *	Called by ip_vs_conn_new.
  */
 static inline void ip_vs_bind_xmit(struct ip_vs_conn *cp)
 {
@@ -334,12 +334,13 @@ static inline void ip_vs_bind_xmit(struct ip_vs_conn *cp)
 
 static inline int ip_vs_dest_totalconns(struct ip_vs_dest *dest)
 {
-	return atomic_read(&dest->activeconns) + atomic_read(&dest->inactconns);
+	return atomic_read(&dest->activeconns)
+		+ atomic_read(&dest->inactconns);
 }
 
 /*
- *  Bind a connection entry with a virtual service destination
- *  Called just after a new connection entry is created.
+ *	Bind a connection entry with a virtual service destination
+ *	Called just after a new connection entry is created.
  */
 static inline void
 ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
@@ -365,11 +366,17 @@ ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
 		  cp->flags, atomic_read(&cp->refcnt),
 		  atomic_read(&dest->refcnt));
 
-	/* Increase the inactive connection counter because it is in TCP
-	   SYNRECV state (inactive) or other protocol inacive state, if
-	   it is not a connection template. */
-	if (cp->cport || (cp->flags & IP_VS_CONN_F_NO_CPORT))
+	/* Update the connection counters */
+	if (cp->cport || (cp->flags & IP_VS_CONN_F_NO_CPORT)) {
+		/* It is a normal connection, so increase the inactive
+		   connection counter because it is in TCP SYNRECV
+		   state (inactive) or other protocol inacive state */
 		atomic_inc(&dest->inactconns);
+	} else {
+		/* It is a persistent connection/template, so increase
+		   the peristent connection counter */
+		atomic_inc(&dest->persistconns);
+	}
 
 	if (dest->u_threshold != 0 &&
 	    ip_vs_dest_totalconns(dest) >= dest->u_threshold)
@@ -378,8 +385,8 @@ ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
 
 
 /*
- *  Unbind a connection entry with its VS destination
- *  Called by the ip_vs_conn_expire function.
+ *	Unbind a connection entry with its VS destination
+ *	Called by the ip_vs_conn_expire function.
  */
 static inline void ip_vs_unbind_dest(struct ip_vs_conn *cp)
 {
@@ -398,16 +405,19 @@ static inline void ip_vs_unbind_dest(struct ip_vs_conn *cp)
 		  cp->flags, atomic_read(&cp->refcnt),
 		  atomic_read(&dest->refcnt));
 
-	/*
-	 * Decrease the inactconns or activeconns counter
-	 * if it is not a connection template.
-	 */
+	/* Update the connection counters */
 	if (cp->cport || (cp->flags & IP_VS_CONN_F_NO_CPORT)) {
+		/* It is a normal connection, so decrease the inactconns
+		   or activeconns counter */
 		if (cp->flags & IP_VS_CONN_F_INACTIVE) {
 			atomic_dec(&dest->inactconns);
 		} else {
 			atomic_dec(&dest->activeconns);
 		}
+	} else {
+		/* It is a persistent connection/template, so decrease
+		   the peristent connection counter */
+		atomic_dec(&dest->persistconns);
 	}
 
 	if (dest->l_threshold != 0) {
@@ -431,9 +441,9 @@ static inline void ip_vs_unbind_dest(struct ip_vs_conn *cp)
 
 
 /*
- *  Checking if the destination of a connection template is available.
- *  If available, return 1, otherwise invalidate this connection
- *  template and return 0.
+ *	Checking if the destination of a connection template is available.
+ *	If available, return 1, otherwise invalidate this connection
+ *	template and return 0.
  */
 int ip_vs_check_template(struct ip_vs_conn *ct)
 {
@@ -538,7 +548,7 @@ void ip_vs_conn_expire_now(struct ip_vs_conn *cp)
 
 
 /*
- *  Create a new connection entry and hash it into the ip_vs_conn_tab.
+ *	Create a new connection entry and hash it into the ip_vs_conn_tab
  */
 struct ip_vs_conn *
 ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
@@ -588,7 +598,7 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 
 	/* Set its state and timeout */
 	cp->state = 0;
-	cp->timeout = 2*HZ;
+	cp->timeout = 3*HZ;
 
 	/* Bind its packet transmitter */
 	ip_vs_bind_xmit(cp);
@@ -844,6 +854,7 @@ int ip_vs_conn_init(void)
 	return 0;
 }
 
+
 void ip_vs_conn_cleanup(void)
 {
 	/* flush all the connection entries first */
