@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.34 2003/03/22 06:31:21 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.35 2003/04/18 09:03:16 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -741,6 +741,7 @@ ip_vs_new_dest(struct ip_vs_service *svc, struct ip_vs_dest_user *udest,
 
 	atomic_set(&dest->activeconns, 0);
 	atomic_set(&dest->inactconns, 0);
+	atomic_set(&dest->persistconns, 0);
 	atomic_set(&dest->refcnt, 0);
 
 	INIT_LIST_HEAD(&dest->d_list);
@@ -1956,6 +1957,7 @@ __ip_vs_get_dest_entries(const struct ip_vs_get_dests *get,
 			entry.l_threshold = dest->l_threshold;
 			entry.activeconns = atomic_read(&dest->activeconns);
 			entry.inactconns = atomic_read(&dest->inactconns);
+			entry.persistconns = atomic_read(&dest->persistconns);
 			ip_vs_copy_stats(&entry.stats, &dest->stats);
 			if (copy_to_user(&uptr->entrytable[count],
 					 &entry, sizeof(entry))) {
