@@ -1,7 +1,7 @@
 /*
- * IP_VS        ftp application module
+ * ip_vs_ftp.c: IPVS ftp application module
  *
- * Version:	$Id: ip_vs_ftp.c,v 1.11 2002/04/03 14:19:01 wensong Exp $
+ * Version:	$Id: ip_vs_ftp.c,v 1.13 2002/09/15 08:14:08 wensong Exp $
  *
  * Authors:	Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -48,7 +48,6 @@
  * First port is set to the default port.
  */
 static int ports[IP_VS_APP_MAX_PORTS] = {21, 0};
-struct ip_vs_app *incarnations[IP_VS_APP_MAX_PORTS];
 
 /*
  *	Debug level
@@ -65,14 +64,14 @@ static int ip_vs_ftp_pasv;
 
 
 static int
-ip_vs_ftp_init_conn(struct ip_vs_app *vapp, struct ip_vs_conn *cp)
+ip_vs_ftp_init_conn(struct ip_vs_app *app, struct ip_vs_conn *cp)
 {
 	return 0;
 }
 
 
 static int
-ip_vs_ftp_done_conn(struct ip_vs_app *vapp, struct ip_vs_conn *cp)
+ip_vs_ftp_done_conn(struct ip_vs_app *app, struct ip_vs_conn *cp)
 {
 	return 0;
 }
@@ -137,7 +136,7 @@ static int ip_vs_ftp_get_addrport(char *data, char *data_limit,
  *   "227 Entering Passive Mode (xxx,xxx,xxx,xxx,ppp,ppp)".
  * xxx,xxx,xxx,xxx is the server address, ppp,ppp is the server port number.
  */
-static int ip_vs_ftp_out(struct ip_vs_app *vapp,
+static int ip_vs_ftp_out(struct ip_vs_app *app,
 			 struct ip_vs_conn *cp, struct sk_buff *skb)
 {
 	struct iphdr *iph;
@@ -152,7 +151,7 @@ static int ip_vs_ftp_out(struct ip_vs_app *vapp,
 	int diff;
 
 	/* Only useful for established sessions */
-	if (cp->state != IP_VS_S_ESTABLISHED)
+	if (cp->state != IP_VS_TCP_S_ESTABLISHED)
 		return 0;
 
 	if (cp->app_data == &ip_vs_ftp_pasv) {
@@ -189,10 +188,6 @@ static int ip_vs_ftp_out(struct ip_vs_app *vapp,
 
 			/* add its controller */
 			ip_vs_control_add(n_cp, cp);
-
-			/* increase dest's inactive connection counter */
-			if (cp->dest)
-				atomic_inc(&cp->dest->inactconns);
 		}
 
 		/*
@@ -219,7 +214,7 @@ static int ip_vs_ftp_out(struct ip_vs_app *vapp,
 		}
 
 		cp->app_data = NULL;
-		ip_vs_conn_listen(n_cp);
+		ip_vs_tcp_conn_listen(n_cp);
 		ip_vs_conn_put(n_cp);
 		return diff;
 	}
@@ -238,7 +233,7 @@ static int ip_vs_ftp_out(struct ip_vs_app *vapp,
  * port, so that the active ftp data connection from the server can reach
  * the client.
  */
-static int ip_vs_ftp_in(struct ip_vs_app *vapp,
+static int ip_vs_ftp_in(struct ip_vs_app *app,
 			struct ip_vs_conn *cp, struct sk_buff *skb)
 {
 	struct iphdr *iph;
@@ -250,7 +245,7 @@ static int ip_vs_ftp_in(struct ip_vs_app *vapp,
 	struct ip_vs_conn *n_cp;
 
 	/* Only useful for established sessions */
-	if (cp->state != IP_VS_S_ESTABLISHED)
+	if (cp->state != IP_VS_TCP_S_ESTABLISHED)
 		return 0;
 
 	/*
@@ -261,7 +256,7 @@ static int ip_vs_ftp_in(struct ip_vs_app *vapp,
 
 	/* Since there may be OPTIONS in the TCP packet and the HLEN is
 	   the length of the header in 32-bit multiples, it is accurate
-	   to caculate data address by th+HLEN*4 */
+	   to calculate data address by th+HLEN*4 */
 	data = data_start = (char *)th + (th->doff << 2);
 	data_limit = skb->tail;
 
@@ -317,16 +312,12 @@ static int ip_vs_ftp_in(struct ip_vs_app *vapp,
 
 		/* add its controller */
 		ip_vs_control_add(n_cp, cp);
-
-		/* increase dest's inactive connection counter */
-		if (cp->dest)
-			atomic_inc(&cp->dest->inactconns);
 	}
 
 	/*
 	 *	Move tunnel to listen state
 	 */
-	ip_vs_conn_listen(n_cp);
+	ip_vs_tcp_conn_listen(n_cp);
 	ip_vs_conn_put(n_cp);
 
 	/* no diff required for incoming packets */
@@ -335,14 +326,17 @@ static int ip_vs_ftp_in(struct ip_vs_app *vapp,
 
 
 static struct ip_vs_app ip_vs_ftp = {
-	{0},			/* n_list */
-	"ftp",			/* name */
-	0,                      /* type */
-	THIS_MODULE,            /* this module */
-	ip_vs_ftp_init_conn,    /* ip_vs_init_conn */
-	ip_vs_ftp_done_conn,    /* ip_vs_done_conn */
-	ip_vs_ftp_out,          /* pkt_out */
-	ip_vs_ftp_in,           /* pkt_in */
+	.name =		"ftp",
+	.type =		IP_VS_APP_TYPE_FTP,
+	.protocol =	IPPROTO_TCP,
+	.module =	THIS_MODULE,
+	.incs_list =	LIST_HEAD_INIT(ip_vs_ftp.incs_list),
+	.init_conn =	ip_vs_ftp_init_conn,
+	.done_conn =	ip_vs_ftp_done_conn,
+	.bind_conn =	NULL,
+	.unbind_conn =	NULL,
+	.pkt_out =	ip_vs_ftp_out,
+	.pkt_in =	ip_vs_ftp_in,
 };
 
 
@@ -351,31 +345,27 @@ static struct ip_vs_app ip_vs_ftp = {
  */
 static int __init ip_vs_ftp_init(void)
 {
-	int i, j;
+	int i, ret;
+	struct ip_vs_app *app = &ip_vs_ftp;
+
+	ret = register_ip_vs_app(app);
+	if (ret)
+		return ret;
 
 	for (i=0; i<IP_VS_APP_MAX_PORTS; i++) {
-		if (ports[i]) {
-			if (!(incarnations[i] =
-			     kmalloc(sizeof(struct ip_vs_app), GFP_KERNEL)))
-				return -ENOMEM;
-
-			memcpy(incarnations[i], &ip_vs_ftp,
-			       sizeof(struct ip_vs_app));
-			if ((j = register_ip_vs_app(incarnations[i],
-						    IPPROTO_TCP,
-						    ports[i]))) {
-				return j;
-			}
-			IP_VS_DBG(1-debug,
-				  "Ftp: loaded support on port[%d] = %d\n",
-				  i, ports[i]);
-		} else {
-			/* To be safe, force the incarnation table entry
-			   to be NULL */
-			incarnations[i] = NULL;
-		}
+		if (!ports[i])
+			continue;
+		ret = register_ip_vs_app_inc(app, app->protocol, ports[i]);
+		if (ret)
+			break;
+		IP_VS_DBG(1-debug, "%s: loaded support on port[%d] = %d\n",
+			  app->name, i, ports[i]);
 	}
-	return 0;
+
+	if (ret)
+		unregister_ip_vs_app(app);
+
+	return ret;
 }
 
 
@@ -384,21 +374,7 @@ static int __init ip_vs_ftp_init(void)
  */
 static void __exit ip_vs_ftp_exit(void)
 {
-	int i, j, k;
-
-	k=0;
-	for (i=0; i<IP_VS_APP_MAX_PORTS; i++) {
-		if (incarnations[i]) {
-			if ((j = unregister_ip_vs_app(incarnations[i]))) {
-				k = j;
-			} else {
-				kfree(incarnations[i]);
-				incarnations[i] = NULL;
-				IP_VS_DBG(1-debug, "Ftp: unloaded support on port[%d] = %d\n",
-					  i, ports[i]);
-			}
-		}
-	}
+	unregister_ip_vs_app(&ip_vs_ftp);
 }
 
 
