@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sync.c,v 1.11 2002/12/19 08:40:52 wensong Exp $
+ * Version:     $Id: ip_vs_sync.c,v 1.12 2003/03/22 06:31:21 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -303,7 +303,7 @@ static void ip_vs_process_message(const char *buffer, const size_t buflen)
 		} else
 			p += SIMPLE_CONN_SIZE;
 
-		atomic_set(&cp->in_pkts, sysctl_ip_vs_sync_threshold);
+		atomic_set(&cp->in_pkts, sysctl_ip_vs_sync_threshold[0]);
 		cp->timeout = IP_VS_SYNC_CONN_TIMEOUT;
 		ip_vs_conn_put(cp);
 
@@ -703,23 +703,30 @@ static int sync_thread(void *startup)
 	DECLARE_WAITQUEUE(wait, current);
 	mm_segment_t oldmm;
 	int state;
+	const char *name;
 
-	MOD_INC_USE_COUNT;
-	daemonize();
+	/* increase the module use count */
+	ip_vs_use_count_inc();
+
+	if (ip_vs_sync_state == IP_VS_STATE_MASTER)
+		name = "ipvs syncmaster";
+	else if (ip_vs_sync_state == IP_VS_STATE_BACKUP)
+		name = "ipvs syncbackup";
+	else {
+		IP_VS_BUG();
+		ip_vs_use_count_dec();
+		return -EINVAL;
+	}
+
+	daemonize(name);
 
 	oldmm = get_fs();
 	set_fs(KERNEL_DS);
 
-	if (ip_vs_sync_state == IP_VS_STATE_MASTER)
-		sprintf(current->comm, "ipvs syncmaster");
-	else if (ip_vs_sync_state == IP_VS_STATE_BACKUP)
-		sprintf(current->comm, "ipvs syncbackup");
-	else IP_VS_BUG();
-
-	spin_lock_irq(&current->sig->siglock);
+	spin_lock_irq(&current->sighand->siglock);
 	siginitsetinv(&current->blocked, 0);
 	recalc_sigpending();
-	spin_unlock_irq(&current->sig->siglock);
+	spin_unlock_irq(&current->sighand->siglock);
 
 	/* set the maximum length of sync message */
 	set_sync_mesg_maxlen(ip_vs_mcast_ifn);
@@ -750,7 +757,9 @@ static int sync_thread(void *startup)
 	IP_VS_INFO("sync thread stopped!\n");
 
 	set_fs(oldmm);
-	MOD_DEC_USE_COUNT;
+
+	/* decrease the module use count */
+	ip_vs_use_count_dec();
 
 	stop_sync = 0;
 	wake_up(&stop_sync_wait);
