@@ -2,7 +2,7 @@
  *      ipvsadm - IP Virtual Server ADMinistration program
  *                for IPVS NetFilter Module in kernel 2.4
  *
- *      Version: $Id: ipvsadm.c,v 1.21 2002/11/30 01:50:37 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.23 2003/04/18 09:03:19 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *               Peter Kese <peter.kese@ijs.si>
@@ -119,7 +119,7 @@
 #include "libipvs/libipvs.h"
 
 #define IPVSADM_VERSION_NO	"v" VERSION
-#define IPVSADM_VERSION_DATE	"2002/11/15"
+#define IPVSADM_VERSION_DATE	"2003/04/16"
 #define IPVSADM_VERSION		IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
 #define MAX_TIMEOUT		(86400*31)	/* 31 days */
@@ -177,8 +177,9 @@ static const char* cmdnames[] = {
 #define OPT_STATS		0x04000
 #define OPT_RATE		0x08000
 #define OPT_THRESHOLDS		0x10000
-#define OPT_SORT		0x20000
-#define NUMBER_OF_OPT		18
+#define OPT_PERSISTENTCONN	0x20000
+#define OPT_SORT		0x40000
+#define NUMBER_OF_OPT		19
 
 static const char* optnames[] = {
 	"numeric",
@@ -198,6 +199,7 @@ static const char* optnames[] = {
 	"stats",
 	"rate",
 	"thresholds",
+	"persistent-conn",
 	"sort",
 };
 
@@ -211,43 +213,44 @@ static const char* optnames[] = {
  */
 static const char commands_v_options[NUMBER_OF_CMD][NUMBER_OF_OPT] =
 {
-	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr  srt */
-/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' ', ' '},
-/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr  -pc  srt */
+/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' ', ' ', ' '},
+/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
 };
 
 /* printing format flags */
-#define FMT_NONE	0x0000
-#define FMT_NUMERIC	0x0001
-#define FMT_RULE	0x0002
-#define FMT_STATS	0x0004
-#define FMT_RATE	0x0008
-#define FMT_THRESHOLDS	0x0010
-#define FMT_SORT	0x0020
+#define FMT_NONE		0x0000
+#define FMT_NUMERIC		0x0001
+#define FMT_RULE		0x0002
+#define FMT_STATS		0x0004
+#define FMT_RATE		0x0008
+#define FMT_THRESHOLDS		0x0010
+#define FMT_PERSISTENTCONN	0x0020
+#define FMT_SORT		0x0040
 
-#define SERVICE_NONE    0x0
-#define SERVICE_ADDR    0x1
-#define SERVICE_PORT    0x2
+#define SERVICE_NONE		0x0000
+#define SERVICE_ADDR		0x0001
+#define SERVICE_PORT		0x0002
 
 /* default scheduler */
-#define DEF_SCHED	"wlc"
+#define DEF_SCHED		"wlc"
 
 /* default multicast interface name */
-#define DEF_MCAST_IFN   "eth0"
+#define DEF_MCAST_IFN		"eth0"
 
-#define CONN_PROC_FILE  "/proc/net/ip_vs_conn"
+#define CONN_PROC_FILE		"/proc/net/ip_vs_conn"
 
 struct ipvs_command_entry {
 	int			cmd;
@@ -378,6 +381,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"stats", '\0', POPT_ARG_NONE, NULL, '7'},
 		{"rate", '\0', POPT_ARG_NONE, NULL, '8'},
 		{"thresholds", '\0', POPT_ARG_NONE, NULL, '9'},
+		{"persistent-conn", '\0', POPT_ARG_NONE, NULL, 'P'},
 		{"sort", '\0', POPT_ARG_NONE, NULL, '0'},
 		{NULL, 0, 0, NULL, 0}
 	};
@@ -432,7 +436,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			ce->daemon.state = IP_VS_STATE_MASTER;
 		else if (!strcmp(optarg, "backup"))
 			ce->daemon.state = IP_VS_STATE_BACKUP;
-		else fail(2, "illegal start_daemon specified");
+		else fail(2, "illegal start-daemon parameter specified");
 		break;
 	case '2':
 		set_command(&ce->cmd, CMD_STOPDAEMON);
@@ -562,6 +566,10 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			set_option(options, OPT_THRESHOLDS);
 			*format |= FMT_THRESHOLDS;
 			break;
+		case 'P':
+			set_option(options, OPT_PERSISTENTCONN);
+			*format |= FMT_PERSISTENTCONN;
+			break;
 		case '0':
 			set_option(options, OPT_SORT);
 			*format |= FMT_SORT;
@@ -651,6 +659,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"stats", 0, 0, '7'},
 		{"rate", 0, 0, '8'},
 		{"thresholds", 0, 0, '9'},
+		{"persistent-conn", 0, 0, 'P'},
 		{"sort", 0, 0, '0'},
 		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
@@ -857,6 +866,10 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			set_option(options, OPT_THRESHOLDS);
 			*format |= FMT_THRESHOLDS;
 			break;
+		case 'P':
+			set_option(options, OPT_PERSISTENTCONN);
+			*format |= FMT_PERSISTENTCONN;
+			break;
 		case '0':
 			set_option(options, OPT_SORT);
 			*format |= FMT_SORT;
@@ -948,6 +961,7 @@ static int process_options(int argc, char **argv, int reading_stdin)
 		if ((options & OPT_CONNECTION ||
 		     options & OPT_TIMEOUT || options & OPT_DAEMON) &&
 		    (options & OPT_STATS ||
+		     options & OPT_PERSISTENTCONN ||
 		     options & OPT_RATE || options & OPT_THRESHOLDS))
 			fail(2, "options conflicts in the list command");
 
@@ -1283,6 +1297,7 @@ static void usage_exit(const char *program, const int exit_status)
 		"  --stats                             output of statistics information\n"
 		"  --rate                              output of rate information\n"
 		"  --thresholds                        output of thresholds information\n"
+		"  --persistent-conn                   output of persistent connection info\n"
 		"  --sort                              sorting output of service/server entries\n"
 		"  --numeric      -n                   numeric output of addresses and ports\n",
 		DEF_SCHED);
@@ -1522,12 +1537,17 @@ static void print_title(unsigned int format)
 		       "  -> RemoteAddress:Port\n",
 		       "Prot LocalAddress:Port",
 		       "CPS", "InPPS", "OutPPS", "InBPS", "OutBPS");
-	else if (format & FMT_THRESHOLDS) {
+	else if (format & FMT_THRESHOLDS)
 		printf("%-33s %-10s %-10s %-10s %-10s\n"
 		       "  -> RemoteAddress:Port\n",
 		       "Prot LocalAddress:Port",
 		       "Uthreshold", "Lthreshold", "ActiveConn", "InActConn");
-	} else if (!(format & FMT_RULE))
+	else if (format & FMT_PERSISTENTCONN)
+		printf("%-33s %-9s %-11s %-10s %-10s\n"
+		       "  -> RemoteAddress:Port\n",
+		       "Prot LocalAddress:Port",
+		       "Weight", "PersistConn", "ActiveConn", "InActConn");
+	else if (!(format & FMT_RULE))
 		printf("Prot LocalAddress:Port Scheduler Flags\n"
 		       "  -> RemoteAddress:Port           Forward Weight ActiveConn InActConn\n");
 }
@@ -1646,6 +1666,10 @@ print_service_entry(ipvs_service_entry_t *se, unsigned int format)
 			printf("  -> %-28s %-10u %-10u %-10u %-10u\n", dname,
 			       e->u_threshold, e->l_threshold,
 			       e->activeconns, e->inactconns);
+		} else if (format & FMT_PERSISTENTCONN) {
+			printf("  -> %-28s %-9u %-11u %-10u %-10u\n", dname,
+			       e->weight, e->persistconns,
+			       e->activeconns, e->inactconns);
 		} else
 			printf("  -> %-28s %-7s %-6d %-10u %-10u\n",
 			       dname, fwd_name(e->conn_flags),
